<?php
    namespace App\Controllers;
    use Psr\Http\Message\ServerRequestInterface as Request;
    use Psr\Http\Message\ResponseInterface as Response;
    use App\Models\Urban;

    class OrderController
    {
        public function updateOrderStatus(Request $request, Response $response, $args)
        {
            $inputData = json_decode(file_get_contents('php://input'), true);
            $errorMessage = "";
            $successMessage = [];
            $params = [
                'storeId' => $inputData['storeId'] ?? '',
                'orderId' => $inputData['orderId'] ?? '',
                'orderStatus' => $inputData['orderStatus'] ?? '',
                'message' => $inputData['message'] ?? '',
                // 'reason_code' => $inputData['reason_code'] ?? '',
                // 'extra' => $inputData['extra'] ?? '',
            ];

            # required fields error
            $errors = [];
            foreach ($params as $key => $value) {
                if (empty($value)) {
                    $errors[] = ucfirst($key) . ' is required';
                }
            }

            if (empty($errors)) {
                $successMessage['new_status'] = $params['orderStatus'];
                $successMessage['message'] = $params['message'];
                # reason code request is used for Cancelled status only
                if($params['orderStatus'] == "Cancelled") {
                    if(empty($inputData['reason_code'])) {
                        $errorMessage = "Reason code is missing";
                    } else {
                        $successMessage['reason_code'] = $inputData['reason_code'];
                    }
                }
                # extra request is used for Acknowledged status only
                if($params['orderStatus'] == "Acknowledged") {
                    $successMessage['extra'] = $inputData['extra'];
                }
            } else {
                $errorMessage = implode('. ', $errors);
            }

            # final error / success message
            if($errorMessage == "") {
                $urbanController = new UrbanController($params['storeId']);
                $updateOrderStatusApi = $urbanController->updateOrderStatusApi($successMessage, $params['orderId']);
                $updateOrderStatusApi = json_decode($updateOrderStatusApi);

                if(isset($updateOrderStatusApi->err_code) || $updateOrderStatusApi->status == "error") {
                    $responseData = [
                        "status" => false, 
                        "message" => $updateOrderStatusApi->message,
                        "response_code" => 400,
                    ];
                } else {
                    
                    $urban = new Urban();
                    # once api call is working then only update prep time to DB
                    if(isset($successMessage['extra']['prep_time_mins'])) {
                        $updateUrbanTatTime = $urban->updateUrbanTatTime($successMessage, $params['storeId'], $params['orderId']);
                    }

                    # update order status to DB
                    $statusMessage = [];
                    $statusMessage['storeId'] = $params['storeId'];
                    $nextButton = ["Acknowledged","Food Ready","Dispatched","Completed","Cancelled"];
                    $nextIndex = array_search($params['orderStatus'], $nextButton) + 1;
                    if($params['orderStatus'] == "Completed" || $params['orderStatus'] == "Cancelled") {
                        $statusMessage['nextButton'] = "";
                    } else {
                        if ($nextIndex !== false && isset($nextButton[$nextIndex])) {
                            $nextValue = $nextButton[$nextIndex];
                        }
                        $statusMessage['nextButton'] = $nextValue;
                    }
                    $statusMessage['orderId'] = $params['orderId'];
                    $statusMessage['newState'] = $params['orderStatus'];

                    $urban->updateUrbanOrders($statusMessage);

                    $responseData = [
                        "status" => true, 
                        "message" => $updateOrderStatusApi,
                        "response_code" => 200,
                    ];
                }
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => $errorMessage,
                    "response_code" => 400,
                ];
            }
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        // public function updateSelfDriverStatus(Request $request, Response $response, $args)
        // {
        //     $inputData = json_decode(file_get_contents('php://input'), true);
        //     $errorMessage = "";
        //     $successMessage = [];
        //     $params = [
        //         'storeId' => $inputData['storeId'] ?? '',
        //         'orderId' => $inputData['orderId'] ?? '',
        //         'riderStatus' => $inputData['riderStatus'] ?? '',
        //         'riderName' => $inputData['riderName'] ?? '',
        //         'riderPhoneNumber' => $inputData['riderPhoneNumber'] ?? '',
        //     ];

        //     # required fields error
        //     $errors = [];
        //     foreach ($params as $key => $value) {
        //         if (empty($value)) {
        //             $errors[] = ucfirst($key) . ' is required';
        //         }
        //     }

        //     if (empty($errors)) {
        //         $successMessage['order_id'] = $params['orderId'];
        //         $successMessage['status'] = $params['riderStatus'];
        //         $successMessage['rider_data']['rider_name'] = $params['riderName'];
        //         $successMessage['rider_data']['rider_phone_number'] = $params['riderPhoneNumber'];
        //     } else {
        //         $errorMessage = implode('. ', $errors);
        //     }

        //     # final error / success message
        //     if($errorMessage == "") {
        //         $urbanController = new UrbanController($params['storeId']);
        //         $updateSelfDriverStatusApi = $urbanController->updateSelfDriverStatusApi($successMessage);
        //         $updateSelfDriverStatusApi = json_decode($updateSelfDriverStatusApi);

        //         // $responseData = [
        //         //     "status" => false, 
        //         //     "message" => $updateSelfDriverStatusApi,
        //         //     "response_code" => 400,
        //         // ];
                
        //         // $response->getBody()->write(json_encode($responseData));
        //         // return $response;

        //         if(isset($updateSelfDriverStatusApi->err_code) || $updateSelfDriverStatusApi->status == "error") {
        //             $responseData = [
        //                 "status" => false, 
        //                 "message" => $updateSelfDriverStatusApi->message,
        //                 "response_code" => 400,
        //             ];
        //         } else {
        //             $responseData = [
        //                 "status" => true, 
        //                 "message" => $updateSelfDriverStatusApi,
        //                 "response_code" => 200,
        //             ];
        //         }
        //     } else {
        //         $responseData = [
        //             "status" => false, 
        //             "message" => $errorMessage,
        //             "response_code" => 400,
        //         ];
        //     }
        //     $response->getBody()->write(json_encode($responseData));
        //     return $response;
        // }
    }
?>