<?php
    namespace App\Controllers;
    use Psr\Http\Message\ServerRequestInterface as Request;
    use Psr\Http\Message\ResponseInterface as Response;
    use App\Models\Store;
    require '../config/env.php';

    class StoreController
    {
        public function createOrUpdateStore(Request $request, Response $response, $args)
        {
            $inputData = json_decode(file_get_contents('php://input'), true);
            $errorMessage = "";
            $successMessage = "";
            $params = [
                'storeId' => $inputData['storeId'] ?? '',
                'store' => $inputData['store'] ?? '',
            ];

            # required fields error
            $errors = [];
            foreach ($params as $key => $value) {
                if (empty($value)) {
                    $errors[] = ucfirst($key) . ' is required';
                }
            }

            if (empty($errors)) {
                $store = new Store();
                $getStore = $store->getStore($params['storeId']);
                $successMessage = $getStore;  
            } else {
                $errorMessage = implode('. ', $errors);
            }

            if($successMessage) {
                // $storeResponse = [];
                $storeResponse['city'] = $successMessage['contacts']->city;
                $storeResponse['name'] = $successMessage['contacts']->name;
                $storeResponse['min_pickup_time'] = $successMessage['cookingTime']['collection'] * 60;
                $storeResponse['min_delivery_time'] = $successMessage['cookingTime']['delivery'] * 60;
                $storeResponse['contact_phone'] = $successMessage['contacts']->mobile;
                $storeResponse['notification_phones'] = array($successMessage['contacts']->phone);
                $storeResponse['ref_id'] = "$params[storeId]";
                $storeResponse['min_order_value'] = floatval($successMessage['min']->delivery_min);
                $storeResponse['hide_from_ui'] = true;
                $storeResponse['address'] = $successMessage['contacts']->address1;
                $storeResponse['notification_emails'] = array($successMessage['contacts']->email);
                $storeResponse['zip_codes'] = $successMessage['contacts']->pincode;
                $storeResponse['geo_longitude'] = floatval($successMessage['postCodeDetail']->longitude);
                $storeResponse['geo_latitude'] = floatval($successMessage['postCodeDetail']->latitude);
                $storeResponse['active'] = true;
                $storeResponse['ordering_enabled'] = true;
                $storeResponse['translations'] = [];
                $storeResponse['excluded_platforms'] = [];
                // $storeResponse['excluded_platforms'] = $_ENV['PLATFORM'];
                foreach($params['store'] as $stores) {
                    $storeData = [
                        'name' => $stores['name'],
                        'url' => $stores['url'],
                        'platform_store_id' => $stores['name'].$params['storeId'] ?? ""
                    ];
                    $storeResponse['platform_data'][] = $storeData;
                }
                $storeResponse['timings'] = $successMessage['shiftTime'];
                $finalStoreResponse['stores'][] = $storeResponse;
            }

            # final error / success message
            if($errorMessage == "") {
                $UrbanController = new UrbanController($params['storeId']);
                $createUpdateStoreApi = $UrbanController->createUpdateStoreApi($finalStoreResponse);
                
                // $responseData = [
                //     "status" => false, 
                //     "message" => $createUpdateStoreApi,
                //     "response_code" => 400,
                // ];
                // $response->getBody()->write(json_encode($responseData));
                // return $response; 
                
                $createUpdateStoreApi = json_decode($createUpdateStoreApi);
                if($createUpdateStoreApi->status == "success") {
                    $responseData = [
                        "status" => true, 
                        "message" => $createUpdateStoreApi,
                        "response_code" => 200,
                    ];
                } else {
                    $responseData = [
                        "status" => false, 
                        "message" => $createUpdateStoreApi->message,
                        "response_code" => 400,
                    ];
                }
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => $errorMessage,
                    "response_code" => 400,
                ];
            }
            $response->getBody()->write(json_encode($responseData));
            return $response;   
        } 

        public function updateStoreStatus(Request $request, Response $response, $args)
        {
            $inputData = json_decode(file_get_contents('php://input'), true);
            $errorMessage = "";
            $params = [
                'storeId' => $inputData['storeId'] ?? '',
                'platforms' => $inputData['platforms'] ?? '',
                'action' => $inputData['action'] ?? '',
            ];

            # required fields error
            $errors = [];
            foreach ($params as $key => $value) {
                if (empty($value)) {
                    $errors[] = ucfirst($key) . ' is required';
                }
            }

            if (empty($errors)) {
                $finalStoreResponse["location_ref_id"] = $params["storeId"];
                $finalStoreResponse["platforms"] = $params["platforms"];
                $finalStoreResponse["action"] = $params["action"];
            } else {
                $errorMessage = implode('. ', $errors);
            }

            # final error / success message
            if($errorMessage == "") {
                $UrbanController = new UrbanController($params['storeId']);
                $updateStoreAction = $UrbanController->updateStoreAction($finalStoreResponse);
                $responseData = [
                    "status" => false, 
                    "message" => $updateStoreAction,
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;   

                $updateStoreAction = json_decode($updateStoreAction);
                if($updateStoreAction->status == "success") {
                    $responseData = [
                        "status" => true, 
                        "message" => $updateStoreAction,
                        "response_code" => 200,
                    ];
                } else {
                    $responseData = [
                        "status" => false, 
                        "message" => $updateStoreAction->message,
                        "response_code" => 400,
                    ];
                }
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => $errorMessage,
                    "response_code" => 400,
                ];
            }
            $response->getBody()->write(json_encode($responseData));
            return $response;   
        } 

        public function testing(Request $request, Response $response, $args)
        {
    
            phpinfo();
    
        }
    }
?>