<?php
    namespace App\Controllers;
    use App\Models\Urban;
    require '../config/env.php';

    class UrbanController
    {
        private $apiUserName;
        private $apiUserKey;
        private $authorization;
        private $storeId;
        private $sourceUrl;

        function __construct($storeId)
        {
            $this->storeId = $storeId; 
            $urban = new Urban();
            $getUrbanCredentials = $urban->getUrbanCredentials($storeId);
            if($getUrbanCredentials != false) {
                $this->apiUserName = $getUrbanCredentials->apiUsername; 
                $this->apiUserKey = $getUrbanCredentials->apiPassword; 
                $this->authorization = "apikey ".$this->apiUserName.":".$this->apiUserKey;  
            }

            if ($_SERVER['SERVER_NAME'] == $_ENV['LOCAL_SERVER_NAME'] || $_SERVER['SERVER_NAME'] == $_ENV['DEV_SERVER_NAME']) {
                $this->sourceUrl = $_ENV['URBAN_TEST_URL'];
            } else {
                $this->sourceUrl = $_ENV['URBAN_LIVE_URL'];
            }
        }

        public function createUpdateStoreApi($data)
        {
            // return $data;

            # error authorization response
            if($this->authorization == "apikey :" || $this->authorization == null) {
                return $this->urbanErrorResponse();
            }

            // API endpoint URL
            $url = $this->sourceUrl.'/external/api/v1/stores/';

            // Headers to be sent in the request
            $headers = [
                'Content-Type: application/json',
                'Authorization: '.$this->authorization
            ];

            // Execute cURL request and get the response
            $response = $this->executeCurlRequest($url, 'POST', json_encode($data), $headers);

            return $response;  
        }  

        public function managingCatalogueApi($data, $flash)
        {
            // return $data;

            # error authorization response
            if($this->authorization == "apikey :" || $this->authorization == null) {
                return $this->urbanErrorResponse();
            }

            // API endpoint URL
            if($flash == 1) {
                $url = $this->sourceUrl.'/external/api/v1/inventory/locations/-1/';
            } else {
                $url = $this->sourceUrl.'/external/api/v1/inventory/locations/'.$this->storeId.'/';
            }
            

            // Headers to be sent in the request
            $headers = [
                'Content-Type: application/json',
                'Authorization: '.$this->authorization 
            ];

            $jsonData = json_encode($data, JSON_UNESCAPED_UNICODE);
            if (json_last_error() !== JSON_ERROR_NONE) {
                $jsonData = $this->removeSpecialCharacters($data);
                $jsonData = json_encode($data, JSON_UNESCAPED_UNICODE);
            }
            // Execute cURL request and get the response
            $response = $this->executeCurlRequest($url, 'POST', $jsonData, $headers);

            return $response;  
        }

    function removeSpecialCharacters(&$data)
    {
        if (is_array($data)) {
            foreach ($data as &$value) {
                $this->removeSpecialCharacters($value);
            }
        } elseif (is_object($data)) {
            foreach ($data as $key => $value) {
                $this->removeSpecialCharacters($data->$key);
            }
        } elseif (is_string($data)) {
            $data = filter_var($data, FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_LOW | FILTER_FLAG_STRIP_HIGH);
        }
    }

        
        public function flashMenuAPI($data)
        {
            // return $data;

            # error authorization response
            if($this->authorization == "apikey :" || $this->authorization == null) {
                return $this->urbanErrorResponse();
            }

            // API endpoint URL
            $url = $this->sourceUrl.'/external/api/v1/inventory/locations/-1/';

            // Headers to be sent in the request
            $headers = [
                'Content-Type: application/json',
                'Authorization: '.$this->authorization 
            ];

            // Execute cURL request and get the response
            $response = $this->executeCurlRequest($url, 'POST', json_encode($data), $headers);

            return $response;  
        }

        public function updateOrderStatusApi($data, $orderId)
        {
            // return $data;

            # error authorization response
            if($this->authorization == "apikey :" || $this->authorization == null) {
                return $this->urbanErrorResponse();
            }

            // API endpoint URL
            $url = $this->sourceUrl.'/external/api/v1/orders/'.$orderId.'/status/';

            // Headers to be sent in the request
            $headers = [
                'Content-Type: application/json',
                'Authorization: '.$this->authorization 
            ];

            // Execute cURL request and get the response
            $response = $this->executeCurlRequest($url, 'PUT', json_encode($data), $headers);

            return $response;  
        }  

        // public function updateStoreAction($data)
        // {
        //     // return $data;

        //     # error authorization response
        //     if($this->authorization == "apikey :" || $this->authorization == null) {
        //         return $this->urbanErrorResponse();
        //     }

        //     // API endpoint URL
        //     $url = $this->sourceUrl.'/hub/api/v1/location/';

        //     // Headers to be sent in the request
        //     $headers = [
        //         'Content-Type: application/json',
        //         'Authorization: '.$this->authorization 
        //     ];

        //     // Execute cURL request and get the response
        //     $response = $this->executeCurlRequest($url, 'POST', json_encode($data), $headers);

        //     return $response;  
        // }  

        // public function updateSelfDriverStatusApi($data)
        // {
        //     // return $data;

        //     # error authorization response
        //     if($this->authorization == "apikey :" || $this->authorization == null) {
        //         return $this->urbanErrorResponse();
        //     }

        //     // API endpoint URL
        //     $url = $this->sourceUrl.'/ext/api/v1/generic/pos/rider-status/';

        //     // Headers to be sent in the request
        //     $headers = [
        //         'Content-Type: application/json',
        //         'Authorization: '.$this->authorization 
        //     ];

        //     // Execute cURL request and get the response
        //     $response = $this->executeCurlRequest($url, 'POST', json_encode($data), $headers);

        //     return $response;  
        // }  

        public function createWebhook($data)
        {
            // return $data;

            # error authorization response
            if($this->authorization == "apikey :" || $this->authorization == null) {
                return $this->urbanErrorResponse();
            }

            // API endpoint URL
            $url = $this->sourceUrl.'/external/api/v1/webhooks/';

            // Headers to be sent in the request
            $headers = [
                'Content-Type: application/json',
                'Authorization: '.$this->authorization 
            ];

            // Execute cURL request and get the response
            $response = $this->executeCurlRequest($url, 'POST', json_encode($data), $headers);

            return $response;  
        }
        
        public function getWebhook($data = "")
        {
            // return $data;

            # error authorization response
            if($this->authorization == "apikey :" || $this->authorization == null) {
                return $this->urbanErrorResponse();
            }

            // API endpoint URL
            $url = $this->sourceUrl.'/external/api/v1/webhooks/'.$data;
            
            // Headers to be sent in the request
            $headers = [
                'Content-Type: application/json',
                'Authorization: '.$this->authorization 
            ];

            // Execute cURL request and get the response
            $response = $this->executeGetCurlRequest($url, 'GET', $headers);

            return $response;  
        }

        private function urbanErrorResponse()
        {
            $data = [
                "status" => false,
                "message" => "Please check urban piper status or credentials"
            ];
            $jsonFormat = json_encode($data);
            return $jsonFormat;
        }

        private function executeCurlRequest($url, $method, $postData, $headers)
        {
            // Initialize cURL session
            $curl = curl_init();

            // Set cURL options using curl_setopt_array
            curl_setopt_array($curl, [
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $method,
                CURLOPT_POSTFIELDS => $postData,
                CURLOPT_HTTPHEADER => $headers,
            ]);

            // Execute cURL request and get the response
            $response = curl_exec($curl);

            // Check for cURL errors
            if (curl_errno($curl)) {
                $response = 'Error: ' . curl_error($curl);
            } 

            // Close cURL session
            curl_close($curl);

            return $response;
        }

        private function executeGetCurlRequest($url, $method, $headers)
        {
            // Initialize cURL session
            $curl = curl_init();

            // Set cURL options using curl_setopt_array
            curl_setopt_array($curl, [
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $method,
                CURLOPT_HTTPHEADER => $headers,
            ]);

            // Execute cURL request and get the response
            $response = curl_exec($curl);

            // Check for cURL errors
            if (curl_errno($curl)) {
                $response = 'Error: ' . curl_error($curl);
            } 

            // Close cURL session
            curl_close($curl);

            return $response;
        }

        function __destruct()
        {
            $this->storeId = null; 
            $this->apiUserName = null; 
            $this->apiUserKey = null; 
            $this->authorization = null;  
            $this->sourceUrl = null;  
        }
    }
?>
