<?php
    namespace App\Controllers;
    use Psr\Http\Message\ServerRequestInterface as Request;
    use Psr\Http\Message\ResponseInterface as Response;
    use App\Helpers\SendEmail;
    use App\Models\Urban;

    class WebhookController
    {
        public function orderPlaced(Request $request, Response $response, $args)
        {
            $errorMessage = "";
            $successMessage = [];
            $payload = $request->getBody();
            $payload = json_decode($payload);

            if($payload == null) 
            {
                $errorMessage = "Webhook Error";
            } 
            else 
            {
                $urban = new Urban();
                $sendEmail = new SendEmail();

                $successMessage['customerDetail'] = json_encode($payload->customer);
                $successMessage['nextStatus'] = json_encode($payload->order->next_states);
                $successMessage['items'] = json_encode($payload->order->items);
                $successMessage['orderDetail'] = json_encode($payload->order->details);
                $successMessage['payment'] = json_encode($payload->order->payment);
                $successMessage['store'] = json_encode($payload->order->store);
                $successMessage['customerName'] = $payload->customer->name;
                $successMessage['customerPhone'] = $payload->customer->phone;
                $successMessage['customerEmail'] = $payload->customer->email;
                $successMessage['businessId'] = $payload->order->details->biz_id;
                $successMessage['storeId'] = $payload->order->store->id;
                $successMessage['platform'] = $payload->order->details->channel;
                $successMessage['orderId'] = $payload->order->details->id;
                $successMessage['orderStatus'] = $payload->order->details->state;
                $successMessage['nextButton'] = $payload->order->next_state;
                $successMessage['total'] = $payload->order->details->order_total;
                $successMessage['orderType'] = $payload->order->details->order_type;
                $successMessage['paymentType'] = $payload->order->payment[0]->option;
                $successMessage['created'] = $payload->order->details->created;
                $successMessage['merchant_ref_id'] = intval($payload->order->store->merchant_ref_id);

                $createOrders = $urban->createUrbanOrders($successMessage);

                # confirm mail 
                $toMail = "ranjith@fusionpos.uk";
                $ccMail = array();
                $mailSub = "Order placed webhook testing";
                $reply = "ranjith@fusionpos.uk";
                $replyName = "Support";
                $mailHtml = json_encode($payload);
                $mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);
            } 

            # final error / success message
            if($errorMessage == "") {
                $responseData = [
                    "status" => true, 
                    "message" => "Success",
                    "response_code" => 200,
                ];
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => $errorMessage,
                    "response_code" => 400,
                ];
            }
            $response->getBody()->write(json_encode($responseData)); 
            return $response;  
        }

        public function orderStatus(Request $request, Response $response, $args)
        {
            $errorMessage = "";
            $successMessage = [];
            $payload = $request->getBody();
            $payload = json_decode($payload);

            if($payload == null) 
            {
                $errorMessage = "Webhook Error";
            } 
            else 
            {
                $urban = new Urban();
                $sendEmail = new SendEmail();

                $successMessage['newState'] = $payload->new_state;
                $successMessage['prevState'] = $payload->prev_state;
                $successMessage['orderId'] = $payload->order_id;
                $successMessage['storeId'] = intval($payload->store_id);

                if($successMessage['newState'] != "Placed" && $successMessage['newState'] != $successMessage['prevState']) 
                {
                    # get urban piper orders detail
                    $getUrbanOrders = $urban->getUrbanOrders($successMessage);
                    // $currentStatus = $getUrbanOrders->order_status;
                    $nextButton = json_decode($getUrbanOrders->next_status);
                    $nextIndex = array_search($successMessage['newState'], $nextButton) + 1;
                    if($successMessage['newState'] == "Completed" || $successMessage['newState'] == "Cancelled") {
                        $successMessage['nextButton'] = "";
                    } else {
                        if ($nextIndex !== false && isset($nextButton[$nextIndex])) {
                            $nextValue = $nextButton[$nextIndex];
                        }
                        $successMessage['nextButton'] = $nextValue;
                    }

                    # update urban piper orders detail
                    $updateUrbanOrders = $urban->updateUrbanOrders($successMessage);
                }

                # confirm mail 
                $toMail = "ranjith@fusionpos.uk";
                $ccMail = array();
                $mailSub = "Order status update webhook testing";
                $reply = "ranjith@fusionpos.uk";
                $replyName = "Support";
                $mailHtml = json_encode($payload);
                $mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);
            } 

            # final error / success message
            if($errorMessage == "") {
                $responseData = [
                    "status" => true, 
                    "message" => "Success",
                    "response_code" => 200,
                ];
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => $errorMessage,
                    "response_code" => 400,
                ];
            }
            $response->getBody()->write(json_encode($responseData)); 
            return $response;  
        }

        public function deliveryStatus(Request $request, Response $response, $args)
        {
            $errorMessage = "";
            $successMessage = [];
            $payload = $request->getBody();
            $payload = json_decode($payload);

            if($payload == null) 
            {
                $errorMessage = "Webhook Error";
            } 
            else 
            {
                $urban = new Urban();
                $sendEmail = new SendEmail();

                $successMessage['deliveryDetail'] = json_encode($payload);
                $successMessage['orderId'] = $payload->order_id;
                $successMessage['storeId'] = intval($payload->store->ref_id);

                # update urban piper driver detail
                $updateUrbanDriver = $urban->updateUrbanDriver($successMessage);

                # confirm mail 
                $toMail = "ranjith@fusionpos.uk";
                $ccMail = array();
                $mailSub = "Delivery status webhook testing";
                $reply = "ranjith@fusionpos.uk";
                $replyName = "Support";
                $mailHtml = json_encode($payload);
                $mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);
            } 

            # final error / success message
            if($errorMessage == "") {
                $responseData = [
                    "status" => true, 
                    "message" => "Success",
                    "response_code" => 200,
                ];
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => $errorMessage,
                    "response_code" => 400,
                ];
            }
            $response->getBody()->write(json_encode($responseData)); 
            return $response;  
        }

        public function catalogueData(Request $request, Response $response, $args)
        {
            $errorMessage = "";
            $successMessage = [];
            $payload = $request->getBody();
            $payload = json_decode($payload);

            if($payload == null) 
            {
                $errorMessage = "Webhook Error";
            } 
            else 
            {
                $sendEmail = new SendEmail();
                
                # confirm mail 
                $toMail = "ranjith@fusionpos.uk";
                $ccMail = array();
                $mailSub = "Catalogue Data webhook testing";
                $reply = "ranjith@fusionpos.uk";
                $replyName = "Support";
                $mailHtml = json_encode($payload);
                $mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);
            } 

            # final error / success message
            if($errorMessage == "") {
                $responseData = [
                    "status" => true, 
                    "message" => "Success",
                    "response_code" => 200,
                ];
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => $errorMessage,
                    "response_code" => 400,
                ];
            }
            $response->getBody()->write(json_encode($responseData)); 
            return $response;  
        }
    }
?>