<?php
namespace App\Models;
require '../config/env.php';

class Store extends DB
{
    public function getStore($clientId)
    {
        $storeDetail = [];
        $storeDetail['contacts'] = $this->getContacts($clientId);
        $storeDetail['cookingTime'] = $this->getCookingTime($clientId);
        $storeDetail['min'] = $this->getMin($clientId);
        $getPostCodeDetail = $this->getPostCodeDetail($storeDetail['contacts']->pincode);
        $storeDetail['postCodeDetail'] = json_decode($getPostCodeDetail);
        $storeDetail['shiftTime'] = $this->getBusinessHours($clientId);
        return $storeDetail;
    }  

    public function getBusinessHours($clientId)
    {
        $shiftTime = $this->getAllShiftTime($clientId);
        $day = ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"];
        $timings = [];

        foreach ($shiftTime as $shift) {
            if ($shift->collection == 1) {
                $shiftData = [];
                $shiftData['day'] = $day[$shift->day - 1];
                $shiftData['slots'] = [];

                if ($shift->collection_closeat < "08:00") {
                    # if shift closed time is minimum of 8, its time added to next date
                    $slate = [];
                    # check open time is greater then 8o clock
                    if($shift->collection_openat >= "08:00") {
                        $slate['start_time'] = $shift->collection_openat . ":00";
                        $slate['end_time'] = "23:59:00";
                    }

                    if ($shift->day == "7") {
                        # if shift day is 7, this is only applicable for sunday shifts
                        $timingData = array_filter($timings, function ($timing) {
                            return strtolower($timing['day']) == 'monday';
                        });

                        if (!empty($timingData)) {
                            $nextSlate = [];
                            if($shift->collection_openat < "08:00") {
                                $nextSlate['start_time'] = $shift->collection_openat . ":00";
                            } else {
                                $nextSlate['start_time'] = "00:00:00";
                            }
                            $nextSlate['end_time'] = $shift->collection_closeat . ":00";

                            foreach ($timingData as $key => $timing) {
                                if (isset($timing['slots'])) {
                                    $timings[$key]['slots'][] = $nextSlate;
                                    break;
                                }
                            }
                        }
                    } else {
                        # this is applicable for either sunday shifts
                        $filteredDay = $day[$shift->day] ?? null;
                        if ($filteredDay !== null) {
                            $timingData = array_filter($timings, function ($timing) use ($filteredDay) {
                                return strtolower($timing['day']) == $filteredDay;
                            });
                        } else {
                            $timingData = [];
                        }

                        if (!empty($timingData)) {
                            $nextSlate = [];
                            if($shift->collection_openat < "08:00") {
                                $nextSlate['start_time'] = $shift->collection_openat . ":00";
                            } else {
                                $nextSlate['start_time'] = "00:00:00";
                            }
                            $nextSlate['end_time'] = $shift->collection_closeat . ":00";

                            foreach ($timingData as $key => $timing) {
                                if (isset($timing['slots'])) {
                                    $timings[$key]['slots'][] = $nextSlate;
                                    break;
                                }
                            }
                        } else {
                            $slots = [
                                'day' => $filteredDay,
                                'slots' => []
                            ];
                            if($shift->collection_openat < "08:00") {
                                $nextSlate['start_time'] = $shift->collection_openat . ":00";
                            } else {
                                $nextSlate['start_time'] = "00:00:00";
                            }
                            $nextSlate['end_time'] = $shift->collection_closeat . ":00";
                            $slots['slots'][] = $nextSlate;
                            $timings[] = $slots;
                        }
                    }
                } else {
                    # if shift closed time is maximum of 8, its time added to next date
                    $slate = [
                        'start_time' => $shift->collection_openat . ":00",
                        'end_time' => $shift->collection_closeat . ":00"
                    ];
                }

                if(!empty($slate)) {
                    $shiftData['slots'][] = $slate;
                }
                $timings[] = $shiftData;
            }
        }

        # combine the all same day slots
        $mergedTimings = [];
        foreach ($timings as $timing) {
            $dayKey = strtolower($timing['day']);

            if (isset($mergedTimings[$dayKey])) {
                $mergedTimings[$dayKey]['slots'] = array_merge($mergedTimings[$dayKey]['slots'], $timing['slots']);
            } else {
                $mergedTimings[$dayKey] = $timing;
            }
        }
        $mergedTimings = array_values($mergedTimings);
        return $mergedTimings;
    }

    
    public function getContacts($clientId)
    {
        $client = new Client($clientId);
        $contact = $client->query("CALL getContacts()");
        $client->connectionClose();
        return $contact;
    }

    public function getCookingTime($clientId)
    {
        $record = array();
        $client = new Client($clientId);
        $cooking_time = $client->query("call getCollectionTime()");
        $client->connectionClose();

        if($cooking_time->collection || $cooking_time->delivery)
        {
            $record['delivery'] = $cooking_time->delivery;
            $record['collection'] = $cooking_time->collection;
        }
        else
        {
            $day = date('w');
            if($day == 0)
            {
                $col_cooking = "collection7";
                $del_cooking = "delivery7";
            } 
            else 
            {
                $col_cooking = "collection".$day;
                $del_cooking = "delivery".$day;
            }
            $record['collection'] = $cooking_time->$col_cooking;
            $record['delivery'] = $cooking_time->$del_cooking;
        }

        return $record;
    }

    public function getMin($clientId)
    {
        $client = new Client($clientId);
        $getMin = $client->query("CALL getMin()");
        $client->connectionClose();
        return $getMin;
    }
    
    public function getPostCodeDetail($post_code)
	{
		$data = array('postcode' => $post_code);
		$url  = 'https://postcode.eposapi.co.uk/addressbypostcode.php';
		$fields = '';
		foreach ($data as $key => $value) 
		{
            $fields .= $key . '=' . $value . '&';
        }
        rtrim($fields, '&');
        $post = curl_init();
        curl_setopt($post, CURLOPT_URL, $url);
        curl_setopt($post, CURLOPT_POST, count($data));
        curl_setopt($post, CURLOPT_POSTFIELDS, $fields);
        curl_setopt($post, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($post);
        $err    = curl_error($post);
        curl_close($post);
        return $result;
	}

    public function getAllShiftTime($client_id)
    {
        $client = new Client($client_id);
        $shift = $client->queryAll("call getAllShiftWithDay()");
        $client->connectionClose();
        return $shift;
    }
}