<?php
    use Psr\Http\Message\ServerRequestInterface as Request;
    use Psr\Http\Message\ResponseInterface as Response;
    use Slim\Factory\AppFactory;
    $app = AppFactory::create();

    // Add middleware to add common header to all responses
    $app->add(function ($request, $handler) {
		if ($request->getMethod() === 'OPTIONS') {
			$response = new \Slim\Psr7\Response();
			$response = $response->withHeader('Access-Control-Allow-Origin', '*')
				->withHeader('Access-Control-Allow-Methods', 'POST')
				->withHeader('Access-Control-Allow-Headers', 'Content-Type');
			return $response;
		}
	
		$response = $handler->handle($request);
		$response = $response->withStatus(200)
			->withHeader('Access-Control-Allow-Origin', '*')
			->withHeader('Content-Type', 'application/json');
		return $response;
	});

    
	// error middleware
    $errorMiddleware = $app->addErrorMiddleware(true, true, true);
    $errorMiddleware->setDefaultErrorHandler(function ($request, $exception, $displayErrorDetails) {
        // Define the log directory
        $logDirectory = __DIR__ . '/../logs/';
        
        // Create the log directory if it doesn't exist
        if (!file_exists($logDirectory)) {
            mkdir($logDirectory, 0755, true); // Create the directory recursively
        }

        // Check if the directory exists or was created successfully
        if (is_dir($logDirectory) && is_writable($logDirectory)) {
            // Create a unique log file based on timestamp
            $logFile = $logDirectory . 'error_' . date('Y-m-d_H-i-s') . '.log';

            // Log the exception to the dynamically created log file
            $logMessage = '[' . date('Y-m-d H:i:s') . '] ' . $exception . PHP_EOL;
            file_put_contents($logFile, $logMessage, FILE_APPEND | LOCK_EX);

            // Handle the exception and return a response
            $statusCode = $exception->getCode() ?: 500;
            $errorMessage = ($displayErrorDetails) ? $exception->getMessage() : 'An error occurred.';
            $response = new \Slim\Psr7\Response();
            $response->getBody()->write("Error: " . $errorMessage);
            return $response->withStatus($statusCode);
        } else {
            $response = new \Slim\Psr7\Response();
            $response->getBody()->write("Error: Log directory is not accessible.");
            return $response->withStatus(500);
        }
    });

    # empty response
	$app->get('/', function (Request $request, Response $response, $args) {
		$response->getBody()->write("Welcome to FusionPos");
		return $response;
	});
	$app->post('/', function (Request $request, Response $response, $args) {
		$response->getBody()->write("Welcome to FusionPos");
		return $response;
	});
    
    // routes
    $app->get('/testing', 'App\Controllers\StoreController:testing');

    # create or update stores and menu controllers
    $app->post('/createOrUpdateStore', 'App\Controllers\StoreController:createOrUpdateStore');
    $app->post('/createOrUpdateMenu', 'App\Controllers\MenuController:createOrUpdateMenu');

    # order Status update controllers
    $app->post('/updateOrderStatus', 'App\Controllers\OrderController:updateOrderStatus');

    # self driver Status update controllers
    // $app->post('/updateSelfDriverStatus', 'App\Controllers\OrderController:updateSelfDriverStatus');

    # web-hook creation controllers
    $app->post('/createBulkWebhooks', 'App\Controllers\SetupWebhookController:createBulkWebhooks');

    # Update Store status
    // $app->post('/updateStoreStatus', 'App\Controllers\StoreController:updateStoreStatus');

    # webhook controllers
    $app->post('/orderPlaced', 'App\Controllers\WebhookController:orderPlaced');
    $app->post('/orderStatus', 'App\Controllers\WebhookController:orderStatus');
    $app->post('/deliveryStatus', 'App\Controllers\WebhookController:deliveryStatus');
    $app->post('/catalogueData', 'App\Controllers\WebhookController:catalogueData');

    # order status api
    $app->post('/getOrderList', 'App\Controllers\OrderListController:getOrderList');
    $app->post('/getNewOrderCount', 'App\Controllers\OrderListController:getNewOrderCount');
    $app->post('/getSeparateOrderDetail', 'App\Controllers\OrderListController:getSeparateOrderDetail');
    $app->post('/updateFusionOrderStatus', 'App\Controllers\OrderListController:updateFusionOrderStatus');
    $app->post('/assignFusionOrderDriver', 'App\Controllers\OrderListController:assignFusionOrderDriver');
    $app->post('/hideOrderStatus', 'App\Controllers\OrderListController:hideOrderStatus');

    $app->run();
